#include <stdlib.h>
#include <net/if.h>
#include <linux/ethtool.h>
#include <string.h>

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"
#include "ethtool.h"

MODULE = Linux::Ethtool		PACKAGE = Linux::Ethtool

unsigned int
get_link(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GLINK;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
get_rx_csum(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GRXCSUM;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_rx_csum(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_SRXCSUM;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

unsigned int
get_tx_csum(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GTXCSUM;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_tx_csum(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_STXCSUM;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

unsigned int
get_sg(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GSG;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_sg(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_SSG;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

unsigned int
get_tso(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GTSO;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_tso(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_STSO;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

unsigned int
get_ufo(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GUFO;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_ufo(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_SUFO;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

unsigned int
get_gso(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GGSO;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_gso(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_SGSO;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

unsigned int
get_gro(dev)
	const char *dev
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd = ETHTOOL_GGRO;
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = (value.data ? 1 : 0);
		}
		else{
			XSRETURN_UNDEF;
		}
	OUTPUT:
		RETVAL

unsigned int
set_gro(dev, enable)
	const char *dev
	unsigned int enable
	
	CODE:
		if(strlen(dev) >= IFNAMSIZ)
		{
			errno = ENAMETOOLONG;
			XSRETURN_UNDEF;
		}
		
		struct ethtool_value value;
		value.cmd  = ETHTOOL_SGRO;
		value.data = (enable ? 1 : 0);
		
		if(_do_ioctl(dev, &value))
		{
			RETVAL = 1;
		}
		else{
			RETVAL = 0;
		}
	OUTPUT:
		RETVAL

